package jalview.schemes;

import jalview.binding.JalviewUserColours;

import java.awt.Color;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;

import org.exolab.castor.xml.Unmarshaller;

public class ColourSchemeLoader
{

  /**
   * Loads a user defined colour scheme from file. The file should contain a
   * definition of residue colours in XML format as defined in
   * JalviewUserColours.xsd.
   * 
   * @param filePath
   * 
   * @return
   */
  public static UserColourScheme loadColourScheme(String filePath)
  {
    UserColourScheme ucs = null;
    Color[] newColours = null;
    File file = new File(filePath);
    try
    {
      InputStreamReader in = new InputStreamReader(
              new FileInputStream(file), "UTF-8");
  
      jalview.schemabinding.version2.JalviewUserColours jucs = new jalview.schemabinding.version2.JalviewUserColours();
  
      org.exolab.castor.xml.Unmarshaller unmar = new org.exolab.castor.xml.Unmarshaller(
              jucs);
      jucs = (jalview.schemabinding.version2.JalviewUserColours) unmar
              .unmarshal(in);
  
      /*
       * non-case-sensitive colours are for 20 amino acid codes,
       * B, Z, X and Gap
       * optionally, lower-case alternatives for all except Gap
       */
      newColours = new Color[24];
      Color[] lowerCase = new Color[23];
      boolean caseSensitive = false;
  
      String name;
      int index;
      for (int i = 0; i < jucs.getColourCount(); i++)
      {
        name = jucs.getColour(i).getName();
        if (ResidueProperties.aa3Hash.containsKey(name))
        {
          index = ResidueProperties.aa3Hash.get(name).intValue();
        }
        else
        {
          index = ResidueProperties.aaIndex[name.charAt(0)];
        }
        if (index == -1)
        {
          continue;
        }
  
        Color color = new Color(Integer.parseInt(jucs.getColour(i)
                .getRGB(), 16));
        if (name.toLowerCase().equals(name))
        {
          caseSensitive = true;
          lowerCase[index] = color;
        }
        else
        {
          newColours[index] = color;
        }
      }
  
      /*
       * instantiate the colour scheme
       */
      ucs = new UserColourScheme(newColours);
      ucs.setName(jucs.getSchemeName());
      if (caseSensitive)
      {
        ucs.setLowerCaseColours(lowerCase);
      }
    } catch (Exception ex)
    {
      // Could be old Jalview Archive format
      try
      {
        InputStreamReader in = new InputStreamReader(new FileInputStream(
                file), "UTF-8");
  
        jalview.binding.JalviewUserColours jucs = new jalview.binding.JalviewUserColours();
  
        jucs = JalviewUserColours.unmarshal(in);
  
        newColours = new Color[jucs.getColourCount()];
  
        for (int i = 0; i < 24; i++)
        {
          newColours[i] = new Color(Integer.parseInt(jucs.getColour(i)
                  .getRGB(), 16));
        }
        ucs = new UserColourScheme(newColours);
        ucs.setName(jucs.getSchemeName());
      } catch (Exception ex2)
      {
        ex2.printStackTrace();
      }
  
      if (newColours == null)
      {
        System.out.println("Error loading User ColourFile\n" + ex);
      }
    }
  
    return ucs;
  }

}
